# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _socketstream
else:
    import _socketstream

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _socketstream.SWIG_PyInstanceMethod_New
_swig_new_static_method = _socketstream.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.mesh
import mfem._ser.matrix
import mfem._ser.vector
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.operators
import mfem._ser.sort_pairs
import mfem._ser.ncmesh
import mfem._ser.gridfunc
import mfem._ser.coefficient
import mfem._ser.globals
import mfem._ser.intrules
import mfem._ser.sparsemat
import mfem._ser.densemat
import mfem._ser.eltrans
import mfem._ser.fe
import mfem._ser.geom
import mfem._ser.fespace
import mfem._ser.fe_coll
import mfem._ser.lininteg
import mfem._ser.handle
import mfem._ser.restriction
import mfem._ser.element
import mfem._ser.table
import mfem._ser.hash
import mfem._ser.bilininteg
import mfem._ser.linearform
import mfem._ser.vertex
import mfem._ser.vtk
class socketbuf(object):
    r"""Proxy of C++ mfem::socketbuf class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(socketbuf self) -> socketbuf
        __init__(socketbuf self, int sd) -> socketbuf
        __init__(socketbuf self, char const [] hostname, int port) -> socketbuf
        """
        _socketstream.socketbuf_swiginit(self, _socketstream.new_socketbuf(*args))

    def attach(self, sd):
        r"""attach(socketbuf self, int sd) -> int"""
        return _socketstream.socketbuf_attach(self, sd)
    attach = _swig_new_instance_method(_socketstream.socketbuf_attach)

    def detach(self):
        r"""detach(socketbuf self) -> int"""
        return _socketstream.socketbuf_detach(self)
    detach = _swig_new_instance_method(_socketstream.socketbuf_detach)

    def open(self, hostname, port):
        r"""open(socketbuf self, char const [] hostname, int port) -> int"""
        return _socketstream.socketbuf_open(self, hostname, port)
    open = _swig_new_instance_method(_socketstream.socketbuf_open)

    def close(self):
        r"""close(socketbuf self) -> int"""
        return _socketstream.socketbuf_close(self)
    close = _swig_new_instance_method(_socketstream.socketbuf_close)

    def getsocketdescriptor(self):
        r"""getsocketdescriptor(socketbuf self) -> int"""
        return _socketstream.socketbuf_getsocketdescriptor(self)
    getsocketdescriptor = _swig_new_instance_method(_socketstream.socketbuf_getsocketdescriptor)

    def is_open(self):
        r"""is_open(socketbuf self) -> bool"""
        return _socketstream.socketbuf_is_open(self)
    is_open = _swig_new_instance_method(_socketstream.socketbuf_is_open)
    __swig_destroy__ = _socketstream.delete_socketbuf

# Register socketbuf in _socketstream:
_socketstream.socketbuf_swigregister(socketbuf)

class socketstream(object):
    r"""Proxy of C++ mfem::socketstream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    secure_default = _socketstream.socketstream_secure_default
    

    def __init__(self, *args):
        r"""
        __init__(socketstream self, bool secure=secure_default) -> socketstream
        __init__(socketstream self, socketbuf buf) -> socketstream
        __init__(socketstream self, int s, bool secure=secure_default) -> socketstream
        __init__(socketstream self, char const [] hostname, int port, bool secure=secure_default) -> socketstream
        """
        _socketstream.socketstream_swiginit(self, _socketstream.new_socketstream(*args))

    def rdbuf(self):
        r"""rdbuf(socketstream self) -> socketbuf"""
        return _socketstream.socketstream_rdbuf(self)
    rdbuf = _swig_new_instance_method(_socketstream.socketstream_rdbuf)

    def open(self, hostname, port):
        r"""open(socketstream self, char const [] hostname, int port) -> int"""
        return _socketstream.socketstream_open(self, hostname, port)
    open = _swig_new_instance_method(_socketstream.socketstream_open)

    def close(self):
        r"""close(socketstream self) -> int"""
        return _socketstream.socketstream_close(self)
    close = _swig_new_instance_method(_socketstream.socketstream_close)

    def is_open(self):
        r"""is_open(socketstream self) -> bool"""
        return _socketstream.socketstream_is_open(self)
    is_open = _swig_new_instance_method(_socketstream.socketstream_is_open)
    __swig_destroy__ = _socketstream.delete_socketstream

    def precision(self, *args):
        r"""
        precision(socketstream self, int const p) -> int
        precision(socketstream self) -> int
        """
        return _socketstream.socketstream_precision(self, *args)
    precision = _swig_new_instance_method(_socketstream.socketstream_precision)

    def send_solution(self, mesh, gf):
        r"""send_solution(socketstream self, Mesh mesh, GridFunction gf)"""
        return _socketstream.socketstream_send_solution(self, mesh, gf)
    send_solution = _swig_new_instance_method(_socketstream.socketstream_send_solution)

    def send_text(self, ostr):
        r"""send_text(socketstream self, char const [] ostr)"""
        return _socketstream.socketstream_send_text(self, ostr)
    send_text = _swig_new_instance_method(_socketstream.socketstream_send_text)

    def flush(self):
        r"""flush(socketstream self)"""
        return _socketstream.socketstream_flush(self)
    flush = _swig_new_instance_method(_socketstream.socketstream_flush)

    def __lshift__(self, *args):
        r"""
        __lshift__(socketstream self, char const [] ostr) -> socketstream
        __lshift__(socketstream self, int const x) -> socketstream
        __lshift__(socketstream self, Mesh mesh) -> socketstream
        __lshift__(socketstream self, GridFunction gf) -> socketstream
        """
        return _socketstream.socketstream___lshift__(self, *args)
    __lshift__ = _swig_new_instance_method(_socketstream.socketstream___lshift__)

    def endline(self):
        r"""endline(socketstream self) -> socketstream"""
        return _socketstream.socketstream_endline(self)
    endline = _swig_new_instance_method(_socketstream.socketstream_endline)

# Register socketstream in _socketstream:
_socketstream.socketstream_swigregister(socketstream)

class socketserver(object):
    r"""Proxy of C++ mfem::socketserver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, port, backlog=4):
        r"""__init__(socketserver self, int port, int backlog=4) -> socketserver"""
        _socketstream.socketserver_swiginit(self, _socketstream.new_socketserver(port, backlog))

    def good(self):
        r"""good(socketserver self) -> bool"""
        return _socketstream.socketserver_good(self)
    good = _swig_new_instance_method(_socketstream.socketserver_good)

    def close(self):
        r"""close(socketserver self) -> int"""
        return _socketstream.socketserver_close(self)
    close = _swig_new_instance_method(_socketstream.socketserver_close)

    def accept(self, *args):
        r"""
        accept(socketserver self) -> int
        accept(socketserver self, socketstream sockstr) -> int
        """
        return _socketstream.socketserver_accept(self, *args)
    accept = _swig_new_instance_method(_socketstream.socketserver_accept)
    __swig_destroy__ = _socketstream.delete_socketserver

# Register socketserver in _socketstream:
_socketstream.socketserver_swigregister(socketserver)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _element
else:
    import _element

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _element.SWIG_PyInstanceMethod_New
_swig_new_static_method = _element.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.globals
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.densemat
import mfem._ser.vector
import mfem._ser.operators
import mfem._ser.matrix
import mfem._ser.geom
import mfem._ser.intrules
import mfem._ser.table
import mfem._ser.hash
class Element(object):
    r"""Proxy of C++ mfem::Element class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    POINT = _element.Element_POINT
    
    SEGMENT = _element.Element_SEGMENT
    
    TRIANGLE = _element.Element_TRIANGLE
    
    QUADRILATERAL = _element.Element_QUADRILATERAL
    
    TETRAHEDRON = _element.Element_TETRAHEDRON
    
    HEXAHEDRON = _element.Element_HEXAHEDRON
    
    WEDGE = _element.Element_WEDGE
    

    def GetType(self):
        r"""GetType(Element self) -> mfem::Element::Type"""
        return _element.Element_GetType(self)
    GetType = _swig_new_instance_method(_element.Element_GetType)

    def GetGeometryType(self):
        r"""GetGeometryType(Element self) -> mfem::Geometry::Type"""
        return _element.Element_GetGeometryType(self)
    GetGeometryType = _swig_new_instance_method(_element.Element_GetGeometryType)

    def GetAttribute(self):
        r"""GetAttribute(Element self) -> int"""
        return _element.Element_GetAttribute(self)
    GetAttribute = _swig_new_instance_method(_element.Element_GetAttribute)

    def SetAttribute(self, attr):
        r"""SetAttribute(Element self, int const attr)"""
        return _element.Element_SetAttribute(self, attr)
    SetAttribute = _swig_new_instance_method(_element.Element_SetAttribute)

    def SetVertices(self, ind):
        r"""SetVertices(Element self, int const * ind)"""
        return _element.Element_SetVertices(self, ind)
    SetVertices = _swig_new_instance_method(_element.Element_SetVertices)

    def GetVertices(self, *args):
        r"""
        GetVertices(Element self, intArray v)
        GetVertices(Element self) -> int
        GetVertices(Element self) -> int const *
        """
        return _element.Element_GetVertices(self, *args)
    GetVertices = _swig_new_instance_method(_element.Element_GetVertices)

    def GetNVertices(self):
        r"""GetNVertices(Element self) -> int"""
        return _element.Element_GetNVertices(self)
    GetNVertices = _swig_new_instance_method(_element.Element_GetNVertices)

    def GetNEdges(self):
        r"""GetNEdges(Element self) -> int"""
        return _element.Element_GetNEdges(self)
    GetNEdges = _swig_new_instance_method(_element.Element_GetNEdges)

    def GetEdgeVertices(self, arg2):
        r"""GetEdgeVertices(Element self, int arg2) -> int const *"""
        return _element.Element_GetEdgeVertices(self, arg2)
    GetEdgeVertices = _swig_new_instance_method(_element.Element_GetEdgeVertices)

    def GetNFaces(self, *args):
        r"""
        GetNFaces(Element self, int & nFaceVertices) -> int
        GetNFaces(Element self) -> int
        """

        if len(args) == 1:
             import warnings
             warnings.warn("Element::GetNFaces(int & nFaceVertices) is deprecated is deprecated",
         	              DeprecationWarning,)


        return _element.Element_GetNFaces(self, *args)


    def GetNFaceVertices(self, fi):
        r"""GetNFaceVertices(Element self, int fi) -> int"""
        return _element.Element_GetNFaceVertices(self, fi)
    GetNFaceVertices = _swig_new_instance_method(_element.Element_GetNFaceVertices)

    def GetFaceVertices(self, fi):
        r"""GetFaceVertices(Element self, int fi) -> int const *"""
        return _element.Element_GetFaceVertices(self, fi)
    GetFaceVertices = _swig_new_instance_method(_element.Element_GetFaceVertices)

    def MarkEdge(self, v_to_v, length):
        r"""MarkEdge(Element self, DSTable v_to_v, int const * length)"""
        return _element.Element_MarkEdge(self, v_to_v, length)
    MarkEdge = _swig_new_instance_method(_element.Element_MarkEdge)

    def NeedRefinement(self, v_to_v):
        r"""NeedRefinement(Element self, mfem::HashTable< mfem::Hashed2 > & v_to_v) -> int"""
        return _element.Element_NeedRefinement(self, v_to_v)
    NeedRefinement = _swig_new_instance_method(_element.Element_NeedRefinement)

    def ResetTransform(self, tr):
        r"""ResetTransform(Element self, int tr)"""
        return _element.Element_ResetTransform(self, tr)
    ResetTransform = _swig_new_instance_method(_element.Element_ResetTransform)

    def PushTransform(self, tr):
        r"""PushTransform(Element self, int tr)"""
        return _element.Element_PushTransform(self, tr)
    PushTransform = _swig_new_instance_method(_element.Element_PushTransform)

    def GetTransform(self):
        r"""GetTransform(Element self) -> unsigned int"""
        return _element.Element_GetTransform(self)
    GetTransform = _swig_new_instance_method(_element.Element_GetTransform)

    def Duplicate(self, m):
        r"""Duplicate(Element self, mfem::Mesh * m) -> Element"""
        return _element.Element_Duplicate(self, m)
    Duplicate = _swig_new_instance_method(_element.Element_Duplicate)
    __swig_destroy__ = _element.delete_Element

    def GetVerticesArray(self):
        r"""GetVerticesArray(Element self) -> PyObject *"""
        return _element.Element_GetVerticesArray(self)
    GetVerticesArray = _swig_new_instance_method(_element.Element_GetVerticesArray)

# Register Element in _element:
_element.Element_swigregister(Element)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pncmesh
else:
    import _pncmesh

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pncmesh.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pncmesh.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pncmesh.MFEM_VERSION

MFEM_VERSION_STRING = _pncmesh.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _pncmesh.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _pncmesh.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _pncmesh.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _pncmesh.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _pncmesh.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _pncmesh.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _pncmesh.MFEM_HYPRE_VERSION

import mfem._par.mesh
import mfem._par.matrix
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.globals
import mfem._par.densemat
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.coefficient
import mfem._par.sparsemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.fespace
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilininteg
import mfem._par.linearform
import mfem._par.communication
import mfem._par.sets
class ParNCMesh(mfem._par.ncmesh.NCMesh):
    r"""Proxy of C++ mfem::ParNCMesh class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(ParNCMesh self, MPI_Comm comm, NCMesh ncmesh, int * part=None) -> ParNCMesh
        __init__(ParNCMesh self, ParNCMesh other) -> ParNCMesh
        """
        _pncmesh.ParNCMesh_swiginit(self, _pncmesh.new_ParNCMesh(*args))
    __swig_destroy__ = _pncmesh.delete_ParNCMesh

    def Refine(self, refinements):
        r"""Refine(ParNCMesh self, RefinementArray refinements)"""
        return _pncmesh.ParNCMesh_Refine(self, refinements)
    Refine = _swig_new_instance_method(_pncmesh.ParNCMesh_Refine)

    def LimitNCLevel(self, max_nc_level):
        r"""LimitNCLevel(ParNCMesh self, int max_nc_level)"""
        return _pncmesh.ParNCMesh_LimitNCLevel(self, max_nc_level)
    LimitNCLevel = _swig_new_instance_method(_pncmesh.ParNCMesh_LimitNCLevel)

    def CheckDerefinementNCLevel(self, deref_table, level_ok, max_nc_level):
        r"""CheckDerefinementNCLevel(ParNCMesh self, Table deref_table, intArray level_ok, int max_nc_level)"""
        return _pncmesh.ParNCMesh_CheckDerefinementNCLevel(self, deref_table, level_ok, max_nc_level)
    CheckDerefinementNCLevel = _swig_new_instance_method(_pncmesh.ParNCMesh_CheckDerefinementNCLevel)

    def Derefine(self, derefs):
        r"""Derefine(ParNCMesh self, intArray derefs)"""
        return _pncmesh.ParNCMesh_Derefine(self, derefs)
    Derefine = _swig_new_instance_method(_pncmesh.ParNCMesh_Derefine)

    def Rebalance(self, custom_partition=None):
        r"""Rebalance(ParNCMesh self, intArray custom_partition=None)"""
        return _pncmesh.ParNCMesh_Rebalance(self, custom_partition)
    Rebalance = _swig_new_instance_method(_pncmesh.ParNCMesh_Rebalance)

    def GetNElements(self):
        r"""GetNElements(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetNElements(self)
    GetNElements = _swig_new_instance_method(_pncmesh.ParNCMesh_GetNElements)

    def GetNGhostVertices(self):
        r"""GetNGhostVertices(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetNGhostVertices(self)
    GetNGhostVertices = _swig_new_instance_method(_pncmesh.ParNCMesh_GetNGhostVertices)

    def GetNGhostEdges(self):
        r"""GetNGhostEdges(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetNGhostEdges(self)
    GetNGhostEdges = _swig_new_instance_method(_pncmesh.ParNCMesh_GetNGhostEdges)

    def GetNGhostFaces(self):
        r"""GetNGhostFaces(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetNGhostFaces(self)
    GetNGhostFaces = _swig_new_instance_method(_pncmesh.ParNCMesh_GetNGhostFaces)

    def GetNGhostElements(self):
        r"""GetNGhostElements(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetNGhostElements(self)
    GetNGhostElements = _swig_new_instance_method(_pncmesh.ParNCMesh_GetNGhostElements)

    def GetSharedVertices(self):
        r"""GetSharedVertices(ParNCMesh self) -> mfem::NCMesh::NCList const &"""
        return _pncmesh.ParNCMesh_GetSharedVertices(self)
    GetSharedVertices = _swig_new_instance_method(_pncmesh.ParNCMesh_GetSharedVertices)

    def GetSharedEdges(self):
        r"""GetSharedEdges(ParNCMesh self) -> mfem::NCMesh::NCList const &"""
        return _pncmesh.ParNCMesh_GetSharedEdges(self)
    GetSharedEdges = _swig_new_instance_method(_pncmesh.ParNCMesh_GetSharedEdges)

    def GetSharedFaces(self):
        r"""GetSharedFaces(ParNCMesh self) -> mfem::NCMesh::NCList const &"""
        return _pncmesh.ParNCMesh_GetSharedFaces(self)
    GetSharedFaces = _swig_new_instance_method(_pncmesh.ParNCMesh_GetSharedFaces)

    def GetSharedList(self, entity):
        r"""GetSharedList(ParNCMesh self, int entity) -> mfem::NCMesh::NCList const &"""
        return _pncmesh.ParNCMesh_GetSharedList(self, entity)
    GetSharedList = _swig_new_instance_method(_pncmesh.ParNCMesh_GetSharedList)

    def GetFaceOrientation(self, index):
        r"""GetFaceOrientation(ParNCMesh self, int index) -> int"""
        return _pncmesh.ParNCMesh_GetFaceOrientation(self, index)
    GetFaceOrientation = _swig_new_instance_method(_pncmesh.ParNCMesh_GetFaceOrientation)

    def GetEntityOwnerId(self, entity, index):
        r"""GetEntityOwnerId(ParNCMesh self, int entity, int index) -> mfem::ParNCMesh::GroupId"""
        return _pncmesh.ParNCMesh_GetEntityOwnerId(self, entity, index)
    GetEntityOwnerId = _swig_new_instance_method(_pncmesh.ParNCMesh_GetEntityOwnerId)

    def GetEntityGroupId(self, entity, index):
        r"""GetEntityGroupId(ParNCMesh self, int entity, int index) -> mfem::ParNCMesh::GroupId"""
        return _pncmesh.ParNCMesh_GetEntityGroupId(self, entity, index)
    GetEntityGroupId = _swig_new_instance_method(_pncmesh.ParNCMesh_GetEntityGroupId)

    def GetGroup(self, id):
        r"""GetGroup(ParNCMesh self, mfem::ParNCMesh::GroupId id) -> mfem::ParNCMesh::CommGroup const &"""
        return _pncmesh.ParNCMesh_GetGroup(self, id)
    GetGroup = _swig_new_instance_method(_pncmesh.ParNCMesh_GetGroup)

    def GroupContains(self, id, rank):
        r"""GroupContains(ParNCMesh self, mfem::ParNCMesh::GroupId id, int rank) -> bool"""
        return _pncmesh.ParNCMesh_GroupContains(self, id, rank)
    GroupContains = _swig_new_instance_method(_pncmesh.ParNCMesh_GroupContains)

    def IsGhost(self, entity, index):
        r"""IsGhost(ParNCMesh self, int entity, int index) -> bool"""
        return _pncmesh.ParNCMesh_IsGhost(self, entity, index)
    IsGhost = _swig_new_instance_method(_pncmesh.ParNCMesh_IsGhost)

    def ElementRank(self, index):
        r"""ElementRank(ParNCMesh self, int index) -> int"""
        return _pncmesh.ParNCMesh_ElementRank(self, index)
    ElementRank = _swig_new_instance_method(_pncmesh.ParNCMesh_ElementRank)

    def GetMyRank(self):
        r"""GetMyRank(ParNCMesh self) -> int"""
        return _pncmesh.ParNCMesh_GetMyRank(self)
    GetMyRank = _swig_new_instance_method(_pncmesh.ParNCMesh_GetMyRank)

    def SendRebalanceDofs(self, old_ndofs, old_element_dofs, old_global_offset, space):
        r"""SendRebalanceDofs(ParNCMesh self, int old_ndofs, Table old_element_dofs, long old_global_offset, FiniteElementSpace space)"""
        return _pncmesh.ParNCMesh_SendRebalanceDofs(self, old_ndofs, old_element_dofs, old_global_offset, space)
    SendRebalanceDofs = _swig_new_instance_method(_pncmesh.ParNCMesh_SendRebalanceDofs)

    def RecvRebalanceDofs(self, elements, dofs):
        r"""RecvRebalanceDofs(ParNCMesh self, intArray elements, mfem::Array< long > & dofs)"""
        return _pncmesh.ParNCMesh_RecvRebalanceDofs(self, elements, dofs)
    RecvRebalanceDofs = _swig_new_instance_method(_pncmesh.ParNCMesh_RecvRebalanceDofs)

    def GetRebalanceOldIndex(self):
        r"""GetRebalanceOldIndex(ParNCMesh self) -> intArray"""
        return _pncmesh.ParNCMesh_GetRebalanceOldIndex(self)
    GetRebalanceOldIndex = _swig_new_instance_method(_pncmesh.ParNCMesh_GetRebalanceOldIndex)

    def GetDerefineOldRanks(self):
        r"""GetDerefineOldRanks(ParNCMesh self) -> intArray"""
        return _pncmesh.ParNCMesh_GetDerefineOldRanks(self)
    GetDerefineOldRanks = _swig_new_instance_method(_pncmesh.ParNCMesh_GetDerefineOldRanks)

    def GetBoundaryClosure(self, bdr_attr_is_ess, bdr_vertices, bdr_edges):
        r"""GetBoundaryClosure(ParNCMesh self, intArray bdr_attr_is_ess, intArray bdr_vertices, intArray bdr_edges)"""
        return _pncmesh.ParNCMesh_GetBoundaryClosure(self, bdr_attr_is_ess, bdr_vertices, bdr_edges)
    GetBoundaryClosure = _swig_new_instance_method(_pncmesh.ParNCMesh_GetBoundaryClosure)

    def Trim(self):
        r"""Trim(ParNCMesh self)"""
        return _pncmesh.ParNCMesh_Trim(self)
    Trim = _swig_new_instance_method(_pncmesh.ParNCMesh_Trim)

    def MemoryUsage(self, with_base=True):
        r"""MemoryUsage(ParNCMesh self, bool with_base=True) -> long"""
        return _pncmesh.ParNCMesh_MemoryUsage(self, with_base)
    MemoryUsage = _swig_new_instance_method(_pncmesh.ParNCMesh_MemoryUsage)

    def PrintMemoryDetail(self, with_base=True):
        r"""PrintMemoryDetail(ParNCMesh self, bool with_base=True) -> int"""
        return _pncmesh.ParNCMesh_PrintMemoryDetail(self, with_base)
    PrintMemoryDetail = _swig_new_instance_method(_pncmesh.ParNCMesh_PrintMemoryDetail)

    def GetDebugMesh(self, debug_mesh):
        r"""GetDebugMesh(ParNCMesh self, Mesh debug_mesh)"""
        return _pncmesh.ParNCMesh_GetDebugMesh(self, debug_mesh)
    GetDebugMesh = _swig_new_instance_method(_pncmesh.ParNCMesh_GetDebugMesh)

# Register ParNCMesh in _pncmesh:
_pncmesh.ParNCMesh_swigregister(ParNCMesh)


def __lt__(a, b):
    r"""__lt__(mfem::NCMesh::MeshId const & a, mfem::NCMesh::MeshId const & b) -> bool"""
    return _pncmesh.__lt__(a, b)
__lt__ = _pncmesh.__lt__

def __eq__(a, b):
    r"""__eq__(mfem::NCMesh::MeshId const & a, mfem::NCMesh::MeshId const & b) -> bool"""
    return _pncmesh.__eq__(a, b)
__eq__ = _pncmesh.__eq__





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _quadrilateral
else:
    import _quadrilateral

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _quadrilateral.SWIG_PyInstanceMethod_New
_swig_new_static_method = _quadrilateral.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.fe
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.geom
import mfem._par.intrules
import mfem._par.densemat
import mfem._par.operators
import mfem._par.matrix
import mfem._par.sparsemat
import mfem._par.element
import mfem._par.globals
import mfem._par.table
import mfem._par.hash
class Quadrilateral(mfem._par.element.Element):
    r"""Proxy of C++ mfem::Quadrilateral class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(Quadrilateral self) -> Quadrilateral
        __init__(Quadrilateral self, int const * ind, int attr=1) -> Quadrilateral
        __init__(Quadrilateral self, int ind1, int ind2, int ind3, int ind4, int attr=1) -> Quadrilateral
        """
        _quadrilateral.Quadrilateral_swiginit(self, _quadrilateral.new_Quadrilateral(*args))

    def GetType(self):
        r"""GetType(Quadrilateral self) -> mfem::Element::Type"""
        return _quadrilateral.Quadrilateral_GetType(self)
    GetType = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetType)

    def SetVertices(self, ind):
        r"""SetVertices(Quadrilateral self, int const * ind)"""
        return _quadrilateral.Quadrilateral_SetVertices(self, ind)
    SetVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_SetVertices)

    def GetVertices(self, *args):
        r"""
        GetVertices(Quadrilateral self, intArray v)
        GetVertices(Quadrilateral self) -> int *
        """
        return _quadrilateral.Quadrilateral_GetVertices(self, *args)
    GetVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetVertices)

    def GetNVertices(self):
        r"""GetNVertices(Quadrilateral self) -> int"""
        return _quadrilateral.Quadrilateral_GetNVertices(self)
    GetNVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetNVertices)

    def GetNEdges(self):
        r"""GetNEdges(Quadrilateral self) -> int"""
        return _quadrilateral.Quadrilateral_GetNEdges(self)
    GetNEdges = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetNEdges)

    def GetEdgeVertices(self, ei):
        r"""GetEdgeVertices(Quadrilateral self, int ei) -> int const *"""
        return _quadrilateral.Quadrilateral_GetEdgeVertices(self, ei)
    GetEdgeVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetEdgeVertices)

    def GetNFaces(self, *args):
        r"""
        GetNFaces(Quadrilateral self, int & nFaceVertices) -> int
        GetNFaces(Quadrilateral self) -> int
        """

        if len(args) == 1:
             import warnings
             warnings.warn("Quadrilateral::GetNFaces(int & nFaceVertices) is deprecated is deprecated",
         	              DeprecationWarning,)


        return _quadrilateral.Quadrilateral_GetNFaces(self, *args)


    def GetNFaceVertices(self, arg2):
        r"""GetNFaceVertices(Quadrilateral self, int arg2) -> int"""
        return _quadrilateral.Quadrilateral_GetNFaceVertices(self, arg2)
    GetNFaceVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetNFaceVertices)

    def GetFaceVertices(self, fi):
        r"""GetFaceVertices(Quadrilateral self, int fi) -> int const *"""
        return _quadrilateral.Quadrilateral_GetFaceVertices(self, fi)
    GetFaceVertices = _swig_new_instance_method(_quadrilateral.Quadrilateral_GetFaceVertices)

    def Duplicate(self, m):
        r"""Duplicate(Quadrilateral self, mfem::Mesh * m) -> Element"""
        return _quadrilateral.Quadrilateral_Duplicate(self, m)
    Duplicate = _swig_new_instance_method(_quadrilateral.Quadrilateral_Duplicate)
    __swig_destroy__ = _quadrilateral.delete_Quadrilateral

# Register Quadrilateral in _quadrilateral:
_quadrilateral.Quadrilateral_swigregister(Quadrilateral)


cvar = _quadrilateral.cvar




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pumi
else:
    import _pumi

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pumi.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pumi.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pumi.MFEM_VERSION

MFEM_VERSION_STRING = _pumi.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _pumi.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _pumi.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _pumi.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _pumi.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _pumi.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _pumi.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _pumi.MFEM_HYPRE_VERSION

class intp(object):
    r"""Proxy of C++ intp class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(intp self) -> intp"""
        _pumi.intp_swiginit(self, _pumi.new_intp())
    __swig_destroy__ = _pumi.delete_intp

    def assign(self, value):
        r"""assign(intp self, int value)"""
        return _pumi.intp_assign(self, value)
    assign = _swig_new_instance_method(_pumi.intp_assign)

    def value(self):
        r"""value(intp self) -> int"""
        return _pumi.intp_value(self)
    value = _swig_new_instance_method(_pumi.intp_value)

    def cast(self):
        r"""cast(intp self) -> int *"""
        return _pumi.intp_cast(self)
    cast = _swig_new_instance_method(_pumi.intp_cast)

    @staticmethod
    def frompointer(t):
        r"""frompointer(int * t) -> intp"""
        return _pumi.intp_frompointer(t)
    frompointer = _swig_new_static_method(_pumi.intp_frompointer)

# Register intp in _pumi:
_pumi.intp_swigregister(intp)

def intp_frompointer(t):
    r"""intp_frompointer(int * t) -> intp"""
    return _pumi.intp_frompointer(t)
intp_frompointer = _pumi.intp_frompointer

class doublep(object):
    r"""Proxy of C++ doublep class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(doublep self) -> doublep"""
        _pumi.doublep_swiginit(self, _pumi.new_doublep())
    __swig_destroy__ = _pumi.delete_doublep

    def assign(self, value):
        r"""assign(doublep self, double value)"""
        return _pumi.doublep_assign(self, value)
    assign = _swig_new_instance_method(_pumi.doublep_assign)

    def value(self):
        r"""value(doublep self) -> double"""
        return _pumi.doublep_value(self)
    value = _swig_new_instance_method(_pumi.doublep_value)

    def cast(self):
        r"""cast(doublep self) -> double *"""
        return _pumi.doublep_cast(self)
    cast = _swig_new_instance_method(_pumi.doublep_cast)

    @staticmethod
    def frompointer(t):
        r"""frompointer(double * t) -> doublep"""
        return _pumi.doublep_frompointer(t)
    frompointer = _swig_new_static_method(_pumi.doublep_frompointer)

# Register doublep in _pumi:
_pumi.doublep_swigregister(doublep)

def doublep_frompointer(t):
    r"""doublep_frompointer(double * t) -> doublep"""
    return _pumi.doublep_frompointer(t)
doublep_frompointer = _pumi.doublep_frompointer

import mfem._par.pgridfunc
import mfem._par.pfespace
import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.pmesh
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets

def ParMesh2ParPumiMesh(pmesh):
    r"""ParMesh2ParPumiMesh(ParMesh pmesh) -> ParPumiMesh"""
    return _pumi.ParMesh2ParPumiMesh(pmesh)
ParMesh2ParPumiMesh = _pumi.ParMesh2ParPumiMesh
class PumiMesh(mfem._par.mesh.Mesh):
    r"""Proxy of C++ mfem::PumiMesh class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, apf_mesh, generate_edges=0, refine=1, fix_orientation=True):
        r"""__init__(PumiMesh self, apf::Mesh2 * apf_mesh, int generate_edges=0, int refine=1, bool fix_orientation=True) -> PumiMesh"""
        _pumi.PumiMesh_swiginit(self, _pumi.new_PumiMesh(apf_mesh, generate_edges, refine, fix_orientation))

    def Load(self, *args):
        r"""
        Load(PumiMesh self, std::istream & input, int generate_edges=0, int refine=1, bool fix_orientation=True)
        Load(PumiMesh self, apf::Mesh2 * apf_mesh, int generate_edges=0, int refine=1, bool fix_orientation=True)
        """
        return _pumi.PumiMesh_Load(self, *args)
    Load = _swig_new_instance_method(_pumi.PumiMesh_Load)
    __swig_destroy__ = _pumi.delete_PumiMesh

# Register PumiMesh in _pumi:
_pumi.PumiMesh_swigregister(PumiMesh)

class ParPumiMesh(mfem._par.pmesh.ParMesh):
    r"""Proxy of C++ mfem::ParPumiMesh class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, comm, apf_mesh, refine=1, fix_orientation=True):
        r"""__init__(ParPumiMesh self, MPI_Comm comm, apf::Mesh2 * apf_mesh, int refine=1, bool fix_orientation=True) -> ParPumiMesh"""
        _pumi.ParPumiMesh_swiginit(self, _pumi.new_ParPumiMesh(comm, apf_mesh, refine, fix_orientation))

    def RotationPUMItoMFEM(self, apf_mesh, tet, elemId):
        r"""RotationPUMItoMFEM(ParPumiMesh self, apf::Mesh2 * apf_mesh, apf::MeshEntity * tet, int elemId) -> int"""
        return _pumi.ParPumiMesh_RotationPUMItoMFEM(self, apf_mesh, tet, elemId)
    RotationPUMItoMFEM = _swig_new_instance_method(_pumi.ParPumiMesh_RotationPUMItoMFEM)

    def ParentXisPUMItoMFEM(self, apf_mesh, tet, elemId, pumi_xi, checkOrientation=True):
        r"""ParentXisPUMItoMFEM(ParPumiMesh self, apf::Mesh2 * apf_mesh, apf::MeshEntity * tet, int elemId, apf::NewArray< apf::Vector3 > & pumi_xi, bool checkOrientation=True) -> IntegrationRule"""
        return _pumi.ParPumiMesh_ParentXisPUMItoMFEM(self, apf_mesh, tet, elemId, pumi_xi, checkOrientation)
    ParentXisPUMItoMFEM = _swig_new_instance_method(_pumi.ParPumiMesh_ParentXisPUMItoMFEM)

    def ParentXisMFEMtoPUMI(self, apf_mesh, elemId, tet, mfem_xi, pumi_xi, checkOrientation=True):
        r"""ParentXisMFEMtoPUMI(ParPumiMesh self, apf::Mesh2 * apf_mesh, int elemId, apf::MeshEntity * tet, IntegrationRule mfem_xi, apf::NewArray< apf::Vector3 > & pumi_xi, bool checkOrientation=True)"""
        return _pumi.ParPumiMesh_ParentXisMFEMtoPUMI(self, apf_mesh, elemId, tet, mfem_xi, pumi_xi, checkOrientation)
    ParentXisMFEMtoPUMI = _swig_new_instance_method(_pumi.ParPumiMesh_ParentXisMFEMtoPUMI)

    def FieldMFEMtoPUMI(self, *args):
        r"""
        FieldMFEMtoPUMI(ParPumiMesh self, apf::Mesh2 * apf_mesh, ParGridFunction grid_vel, ParGridFunction grid_pr, apf::Field * vel_field, apf::Field * pr_field, apf::Field * vel_mag_field)
        FieldMFEMtoPUMI(ParPumiMesh self, apf::Mesh2 * apf_mesh, ParGridFunction grid_pr, apf::Field * pr_field, apf::Field * pr_mag_field)
        """
        return _pumi.ParPumiMesh_FieldMFEMtoPUMI(self, *args)
    FieldMFEMtoPUMI = _swig_new_instance_method(_pumi.ParPumiMesh_FieldMFEMtoPUMI)

    def VectorFieldMFEMtoPUMI(self, apf_mesh, grid_vel, vel_field, vel_mag_field):
        r"""VectorFieldMFEMtoPUMI(ParPumiMesh self, apf::Mesh2 * apf_mesh, ParGridFunction grid_vel, apf::Field * vel_field, apf::Field * vel_mag_field)"""
        return _pumi.ParPumiMesh_VectorFieldMFEMtoPUMI(self, apf_mesh, grid_vel, vel_field, vel_mag_field)
    VectorFieldMFEMtoPUMI = _swig_new_instance_method(_pumi.ParPumiMesh_VectorFieldMFEMtoPUMI)

    def NedelecFieldMFEMtoPUMI(self, apf_mesh, gf, nedelec_field):
        r"""NedelecFieldMFEMtoPUMI(ParPumiMesh self, apf::Mesh2 * apf_mesh, ParGridFunction gf, apf::Field * nedelec_field)"""
        return _pumi.ParPumiMesh_NedelecFieldMFEMtoPUMI(self, apf_mesh, gf, nedelec_field)
    NedelecFieldMFEMtoPUMI = _swig_new_instance_method(_pumi.ParPumiMesh_NedelecFieldMFEMtoPUMI)

    def UpdateMesh(self, AdaptedpMesh):
        r"""UpdateMesh(ParPumiMesh self, ParMesh AdaptedpMesh)"""
        return _pumi.ParPumiMesh_UpdateMesh(self, AdaptedpMesh)
    UpdateMesh = _swig_new_instance_method(_pumi.ParPumiMesh_UpdateMesh)

    def FieldPUMItoMFEM(self, apf_mesh, field, grid):
        r"""FieldPUMItoMFEM(ParPumiMesh self, apf::Mesh2 * apf_mesh, apf::Field * field, ParGridFunction grid)"""
        return _pumi.ParPumiMesh_FieldPUMItoMFEM(self, apf_mesh, field, grid)
    FieldPUMItoMFEM = _swig_new_instance_method(_pumi.ParPumiMesh_FieldPUMItoMFEM)
    __swig_destroy__ = _pumi.delete_ParPumiMesh

# Register ParPumiMesh in _pumi:
_pumi.ParPumiMesh_swigregister(ParPumiMesh)

class GridFunctionPumi(mfem._par.gridfunc.GridFunction):
    r"""Proxy of C++ mfem::GridFunctionPumi class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, m, PumiM, v_num_loc, mesh_order):
        r"""__init__(GridFunctionPumi self, Mesh m, apf::Mesh2 * PumiM, apf::Numbering * v_num_loc, int const mesh_order) -> GridFunctionPumi"""
        _pumi.GridFunctionPumi_swiginit(self, _pumi.new_GridFunctionPumi(m, PumiM, v_num_loc, mesh_order))
    __swig_destroy__ = _pumi.delete_GridFunctionPumi

# Register GridFunctionPumi in _pumi:
_pumi.GridFunctionPumi_swigregister(GridFunctionPumi)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _blockmatrix
else:
    import _blockmatrix

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _blockmatrix.SWIG_PyInstanceMethod_New
_swig_new_static_method = _blockmatrix.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.matrix
import mfem._par.operators
import mfem._par.sparsemat
import mfem._par.densemat
class BlockMatrix(mfem._par.matrix.AbstractSparseMatrix):
    r"""Proxy of C++ mfem::BlockMatrix class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(BlockMatrix self, intArray offsets) -> BlockMatrix
        __init__(BlockMatrix self, intArray row_offsets, intArray col_offsets) -> BlockMatrix
        """
        _blockmatrix.BlockMatrix_swiginit(self, _blockmatrix.new_BlockMatrix(*args))

        from mfem.par import intArray  
        if len(args) == 1:
           if isinstance(args[0], intArray):
               self._offsets = args[0]
        if len(args) == 2:
           if (isinstance(args[0], intArray) and
               isinstance(args[1], intArray)):
               self._offsets = (args[0], args[1])




    def SetBlock(self, i, j, mat):
        r"""SetBlock(BlockMatrix self, int i, int j, SparseMatrix mat)"""
        val = _blockmatrix.BlockMatrix_SetBlock(self, i, j, mat)

        if not hasattr(self, '_linked_mat'):
           self._linked_mat = {}
        self._linked_mat[i, j] = mat


        return val


    def NumRowBlocks(self):
        r"""NumRowBlocks(BlockMatrix self) -> int"""
        return _blockmatrix.BlockMatrix_NumRowBlocks(self)
    NumRowBlocks = _swig_new_instance_method(_blockmatrix.BlockMatrix_NumRowBlocks)

    def NumColBlocks(self):
        r"""NumColBlocks(BlockMatrix self) -> int"""
        return _blockmatrix.BlockMatrix_NumColBlocks(self)
    NumColBlocks = _swig_new_instance_method(_blockmatrix.BlockMatrix_NumColBlocks)

    def GetBlock(self, *args):
        r"""
        GetBlock(BlockMatrix self, int i, int j) -> SparseMatrix
        GetBlock(BlockMatrix self, int i, int j) -> SparseMatrix
        """
        return _blockmatrix.BlockMatrix_GetBlock(self, *args)
    GetBlock = _swig_new_instance_method(_blockmatrix.BlockMatrix_GetBlock)

    def IsZeroBlock(self, i, j):
        r"""IsZeroBlock(BlockMatrix self, int i, int j) -> int"""
        return _blockmatrix.BlockMatrix_IsZeroBlock(self, i, j)
    IsZeroBlock = _swig_new_instance_method(_blockmatrix.BlockMatrix_IsZeroBlock)

    def RowOffsets(self, *args):
        r"""
        RowOffsets(BlockMatrix self) -> intArray
        RowOffsets(BlockMatrix self) -> intArray
        """
        return _blockmatrix.BlockMatrix_RowOffsets(self, *args)
    RowOffsets = _swig_new_instance_method(_blockmatrix.BlockMatrix_RowOffsets)

    def ColOffsets(self, *args):
        r"""
        ColOffsets(BlockMatrix self) -> intArray
        ColOffsets(BlockMatrix self) -> intArray
        """
        return _blockmatrix.BlockMatrix_ColOffsets(self, *args)
    ColOffsets = _swig_new_instance_method(_blockmatrix.BlockMatrix_ColOffsets)

    def RowSize(self, i):
        r"""RowSize(BlockMatrix self, int const i) -> int"""
        return _blockmatrix.BlockMatrix_RowSize(self, i)
    RowSize = _swig_new_instance_method(_blockmatrix.BlockMatrix_RowSize)

    def EliminateRowCol(self, *args):
        r"""
        EliminateRowCol(BlockMatrix self, int rc, mfem::Operator::DiagonalPolicy dpolicy=DIAG_ONE)
        EliminateRowCol(BlockMatrix self, intArray ess_bc_dofs, Vector sol, Vector rhs)
        """
        return _blockmatrix.BlockMatrix_EliminateRowCol(self, *args)
    EliminateRowCol = _swig_new_instance_method(_blockmatrix.BlockMatrix_EliminateRowCol)

    def Finalize(self, *args):
        r"""
        Finalize(BlockMatrix self, int skip_zeros=1)
        Finalize(BlockMatrix self, int skip_zeros, bool fix_empty_rows)
        """
        return _blockmatrix.BlockMatrix_Finalize(self, *args)
    Finalize = _swig_new_instance_method(_blockmatrix.BlockMatrix_Finalize)

    def CreateMonolithic(self):
        r"""CreateMonolithic(BlockMatrix self) -> SparseMatrix"""
        return _blockmatrix.BlockMatrix_CreateMonolithic(self)
    CreateMonolithic = _swig_new_instance_method(_blockmatrix.BlockMatrix_CreateMonolithic)

    def Elem(self, *args):
        r"""
        Elem(BlockMatrix self, int i, int j) -> double
        Elem(BlockMatrix self, int i, int j) -> double const &
        """
        return _blockmatrix.BlockMatrix_Elem(self, *args)
    Elem = _swig_new_instance_method(_blockmatrix.BlockMatrix_Elem)

    def Inverse(self):
        r"""Inverse(BlockMatrix self) -> MatrixInverse"""
        return _blockmatrix.BlockMatrix_Inverse(self)
    Inverse = _swig_new_instance_method(_blockmatrix.BlockMatrix_Inverse)

    def NumNonZeroElems(self):
        r"""NumNonZeroElems(BlockMatrix self) -> int"""
        return _blockmatrix.BlockMatrix_NumNonZeroElems(self)
    NumNonZeroElems = _swig_new_instance_method(_blockmatrix.BlockMatrix_NumNonZeroElems)

    def GetRow(self, row, cols, srow):
        r"""GetRow(BlockMatrix self, int const row, intArray cols, Vector srow) -> int"""
        return _blockmatrix.BlockMatrix_GetRow(self, row, cols, srow)
    GetRow = _swig_new_instance_method(_blockmatrix.BlockMatrix_GetRow)

    def EliminateZeroRows(self, threshold=1e-12):
        r"""EliminateZeroRows(BlockMatrix self, double const threshold=1e-12)"""
        return _blockmatrix.BlockMatrix_EliminateZeroRows(self, threshold)
    EliminateZeroRows = _swig_new_instance_method(_blockmatrix.BlockMatrix_EliminateZeroRows)

    def Mult(self, x, y):
        r"""Mult(BlockMatrix self, Vector x, Vector y)"""
        return _blockmatrix.BlockMatrix_Mult(self, x, y)
    Mult = _swig_new_instance_method(_blockmatrix.BlockMatrix_Mult)

    def AddMult(self, x, y, val=1.):
        r"""AddMult(BlockMatrix self, Vector x, Vector y, double const val=1.)"""
        return _blockmatrix.BlockMatrix_AddMult(self, x, y, val)
    AddMult = _swig_new_instance_method(_blockmatrix.BlockMatrix_AddMult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(BlockMatrix self, Vector x, Vector y)"""
        return _blockmatrix.BlockMatrix_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_blockmatrix.BlockMatrix_MultTranspose)

    def AddMultTranspose(self, x, y, val=1.):
        r"""AddMultTranspose(BlockMatrix self, Vector x, Vector y, double const val=1.)"""
        return _blockmatrix.BlockMatrix_AddMultTranspose(self, x, y, val)
    AddMultTranspose = _swig_new_instance_method(_blockmatrix.BlockMatrix_AddMultTranspose)
    __swig_destroy__ = _blockmatrix.delete_BlockMatrix
    owns_blocks = property(_blockmatrix.BlockMatrix_owns_blocks_get, _blockmatrix.BlockMatrix_owns_blocks_set, doc=r"""owns_blocks : int""")

    def PrintMatlab(self, *args):
        r"""
        PrintMatlab(BlockMatrix self, std::ostream & os=mfem::out)
        PrintMatlab(BlockMatrix self, char const * file, int precision=8)
        """
        return _blockmatrix.BlockMatrix_PrintMatlab(self, *args)
    PrintMatlab = _swig_new_instance_method(_blockmatrix.BlockMatrix_PrintMatlab)

# Register BlockMatrix in _blockmatrix:
_blockmatrix.BlockMatrix_swigregister(BlockMatrix)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _solvers
else:
    import _solvers

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _solvers.SWIG_PyInstanceMethod_New
_swig_new_static_method = _solvers.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _solvers.MFEM_VERSION
MFEM_VERSION_STRING = _solvers.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _solvers.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _solvers.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _solvers.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _solvers.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _solvers.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _solvers.MFEM_VERSION_PATCH
MFEM_HYPRE_VERSION = _solvers.MFEM_HYPRE_VERSION
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.matrix
import mfem._par.sparsemat
import mfem._par.densemat
class IterativeSolverMonitor(object):
    r"""Proxy of C++ mfem::IterativeSolverMonitor class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(IterativeSolverMonitor self) -> IterativeSolverMonitor"""
        _solvers.IterativeSolverMonitor_swiginit(self, _solvers.new_IterativeSolverMonitor())
    __swig_destroy__ = _solvers.delete_IterativeSolverMonitor

    def MonitorResidual(self, it, norm, r, final):
        r"""MonitorResidual(IterativeSolverMonitor self, int it, double norm, Vector r, bool final)"""
        return _solvers.IterativeSolverMonitor_MonitorResidual(self, it, norm, r, final)
    MonitorResidual = _swig_new_instance_method(_solvers.IterativeSolverMonitor_MonitorResidual)

    def MonitorSolution(self, it, norm, x, final):
        r"""MonitorSolution(IterativeSolverMonitor self, int it, double norm, Vector x, bool final)"""
        return _solvers.IterativeSolverMonitor_MonitorSolution(self, it, norm, x, final)
    MonitorSolution = _swig_new_instance_method(_solvers.IterativeSolverMonitor_MonitorSolution)

# Register IterativeSolverMonitor in _solvers:
_solvers.IterativeSolverMonitor_swigregister(IterativeSolverMonitor)

class IterativeSolver(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::IterativeSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def SetRelTol(self, rtol):
        r"""SetRelTol(IterativeSolver self, double rtol)"""
        return _solvers.IterativeSolver_SetRelTol(self, rtol)
    SetRelTol = _swig_new_instance_method(_solvers.IterativeSolver_SetRelTol)

    def SetAbsTol(self, atol):
        r"""SetAbsTol(IterativeSolver self, double atol)"""
        return _solvers.IterativeSolver_SetAbsTol(self, atol)
    SetAbsTol = _swig_new_instance_method(_solvers.IterativeSolver_SetAbsTol)

    def SetMaxIter(self, max_it):
        r"""SetMaxIter(IterativeSolver self, int max_it)"""
        return _solvers.IterativeSolver_SetMaxIter(self, max_it)
    SetMaxIter = _swig_new_instance_method(_solvers.IterativeSolver_SetMaxIter)

    def SetPrintLevel(self, print_lvl):
        r"""SetPrintLevel(IterativeSolver self, int print_lvl)"""
        return _solvers.IterativeSolver_SetPrintLevel(self, print_lvl)
    SetPrintLevel = _swig_new_instance_method(_solvers.IterativeSolver_SetPrintLevel)

    def GetNumIterations(self):
        r"""GetNumIterations(IterativeSolver self) -> int"""
        return _solvers.IterativeSolver_GetNumIterations(self)
    GetNumIterations = _swig_new_instance_method(_solvers.IterativeSolver_GetNumIterations)

    def GetConverged(self):
        r"""GetConverged(IterativeSolver self) -> int"""
        return _solvers.IterativeSolver_GetConverged(self)
    GetConverged = _swig_new_instance_method(_solvers.IterativeSolver_GetConverged)

    def GetFinalNorm(self):
        r"""GetFinalNorm(IterativeSolver self) -> double"""
        return _solvers.IterativeSolver_GetFinalNorm(self)
    GetFinalNorm = _swig_new_instance_method(_solvers.IterativeSolver_GetFinalNorm)

    def SetPreconditioner(self, pr):
        r"""SetPreconditioner(IterativeSolver self, Solver pr)"""
        return _solvers.IterativeSolver_SetPreconditioner(self, pr)
    SetPreconditioner = _swig_new_instance_method(_solvers.IterativeSolver_SetPreconditioner)

    def SetOperator(self, op):
        r"""SetOperator(IterativeSolver self, Operator op)"""
        return _solvers.IterativeSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.IterativeSolver_SetOperator)

    def SetMonitor(self, m):
        r"""SetMonitor(IterativeSolver self, IterativeSolverMonitor m)"""
        return _solvers.IterativeSolver_SetMonitor(self, m)
    SetMonitor = _swig_new_instance_method(_solvers.IterativeSolver_SetMonitor)

    def GetComm(self):
        r"""GetComm(IterativeSolver self) -> MPI_Comm"""
        return _solvers.IterativeSolver_GetComm(self)
    GetComm = _swig_new_instance_method(_solvers.IterativeSolver_GetComm)
    __swig_destroy__ = _solvers.delete_IterativeSolver

# Register IterativeSolver in _solvers:
_solvers.IterativeSolver_swigregister(IterativeSolver)

class OperatorJacobiSmoother(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::OperatorJacobiSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(OperatorJacobiSmoother self, mfem::BilinearForm const & a, intArray ess_tdof_list, double const damping=1.0) -> OperatorJacobiSmoother
        __init__(OperatorJacobiSmoother self, Vector d, intArray ess_tdof_list, double const damping=1.0) -> OperatorJacobiSmoother
        """
        _solvers.OperatorJacobiSmoother_swiginit(self, _solvers.new_OperatorJacobiSmoother(*args))
    __swig_destroy__ = _solvers.delete_OperatorJacobiSmoother

    def Mult(self, x, y):
        r"""Mult(OperatorJacobiSmoother self, Vector x, Vector y)"""
        return _solvers.OperatorJacobiSmoother_Mult(self, x, y)
    Mult = _swig_new_instance_method(_solvers.OperatorJacobiSmoother_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(OperatorJacobiSmoother self, Vector x, Vector y)"""
        return _solvers.OperatorJacobiSmoother_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_solvers.OperatorJacobiSmoother_MultTranspose)

    def SetOperator(self, op):
        r"""SetOperator(OperatorJacobiSmoother self, Operator op)"""
        return _solvers.OperatorJacobiSmoother_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.OperatorJacobiSmoother_SetOperator)

    def Setup(self, diag):
        r"""Setup(OperatorJacobiSmoother self, Vector diag)"""
        return _solvers.OperatorJacobiSmoother_Setup(self, diag)
    Setup = _swig_new_instance_method(_solvers.OperatorJacobiSmoother_Setup)

# Register OperatorJacobiSmoother in _solvers:
_solvers.OperatorJacobiSmoother_swigregister(OperatorJacobiSmoother)

class OperatorChebyshevSmoother(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::OperatorChebyshevSmoother class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(OperatorChebyshevSmoother self, Operator oper_, Vector d, intArray ess_tdof_list, int order, double max_eig_estimate) -> OperatorChebyshevSmoother
        __init__(OperatorChebyshevSmoother self, Operator oper_, Vector d, intArray ess_tdof_list, int order, MPI_Comm comm=MPI_COMM_NULL, int power_iterations=10, double power_tolerance=1e-8) -> OperatorChebyshevSmoother
        """
        _solvers.OperatorChebyshevSmoother_swiginit(self, _solvers.new_OperatorChebyshevSmoother(*args))
    __swig_destroy__ = _solvers.delete_OperatorChebyshevSmoother

    def Mult(self, x, y):
        r"""Mult(OperatorChebyshevSmoother self, Vector x, Vector y)"""
        return _solvers.OperatorChebyshevSmoother_Mult(self, x, y)
    Mult = _swig_new_instance_method(_solvers.OperatorChebyshevSmoother_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(OperatorChebyshevSmoother self, Vector x, Vector y)"""
        return _solvers.OperatorChebyshevSmoother_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_solvers.OperatorChebyshevSmoother_MultTranspose)

    def SetOperator(self, op_):
        r"""SetOperator(OperatorChebyshevSmoother self, Operator op_)"""
        return _solvers.OperatorChebyshevSmoother_SetOperator(self, op_)
    SetOperator = _swig_new_instance_method(_solvers.OperatorChebyshevSmoother_SetOperator)

    def Setup(self):
        r"""Setup(OperatorChebyshevSmoother self)"""
        return _solvers.OperatorChebyshevSmoother_Setup(self)
    Setup = _swig_new_instance_method(_solvers.OperatorChebyshevSmoother_Setup)

# Register OperatorChebyshevSmoother in _solvers:
_solvers.OperatorChebyshevSmoother_swigregister(OperatorChebyshevSmoother)

class SLISolver(IterativeSolver):
    r"""Proxy of C++ mfem::SLISolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(SLISolver self) -> SLISolver
        __init__(SLISolver self, MPI_Comm _comm) -> SLISolver
        """
        _solvers.SLISolver_swiginit(self, _solvers.new_SLISolver(*args))

    def SetOperator(self, op):
        r"""SetOperator(SLISolver self, Operator op)"""
        return _solvers.SLISolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.SLISolver_SetOperator)

    def Mult(self, b, x):
        r"""Mult(SLISolver self, Vector b, Vector x)"""
        return _solvers.SLISolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.SLISolver_Mult)
    __swig_destroy__ = _solvers.delete_SLISolver

# Register SLISolver in _solvers:
_solvers.SLISolver_swigregister(SLISolver)


def SLI(*args):
    r"""
    SLI(Operator A, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, double RTOLERANCE=1e-12, double ATOLERANCE=1e-24)
    SLI(Operator A, Solver B, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, double RTOLERANCE=1e-12, double ATOLERANCE=1e-24)
    """
    return _solvers.SLI(*args)
SLI = _solvers.SLI
class CGSolver(IterativeSolver):
    r"""Proxy of C++ mfem::CGSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(CGSolver self) -> CGSolver
        __init__(CGSolver self, MPI_Comm _comm) -> CGSolver
        """
        _solvers.CGSolver_swiginit(self, _solvers.new_CGSolver(*args))

    def SetOperator(self, op):
        r"""SetOperator(CGSolver self, Operator op)"""
        return _solvers.CGSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.CGSolver_SetOperator)

    def Mult(self, b, x):
        r"""Mult(CGSolver self, Vector b, Vector x)"""
        return _solvers.CGSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.CGSolver_Mult)
    __swig_destroy__ = _solvers.delete_CGSolver

# Register CGSolver in _solvers:
_solvers.CGSolver_swigregister(CGSolver)


def CG(A, b, x, print_iter=0, max_num_iter=1000, RTOLERANCE=1e-12, ATOLERANCE=1e-24):
    r"""CG(Operator A, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, double RTOLERANCE=1e-12, double ATOLERANCE=1e-24)"""
    return _solvers.CG(A, b, x, print_iter, max_num_iter, RTOLERANCE, ATOLERANCE)
CG = _solvers.CG

def PCG(A, B, b, x, print_iter=0, max_num_iter=1000, RTOLERANCE=1e-12, ATOLERANCE=1e-24):
    r"""PCG(Operator A, Solver B, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, double RTOLERANCE=1e-12, double ATOLERANCE=1e-24)"""
    return _solvers.PCG(A, B, b, x, print_iter, max_num_iter, RTOLERANCE, ATOLERANCE)
PCG = _solvers.PCG
class GMRESSolver(IterativeSolver):
    r"""Proxy of C++ mfem::GMRESSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(GMRESSolver self) -> GMRESSolver
        __init__(GMRESSolver self, MPI_Comm _comm) -> GMRESSolver
        """
        _solvers.GMRESSolver_swiginit(self, _solvers.new_GMRESSolver(*args))

    def SetKDim(self, dim):
        r"""SetKDim(GMRESSolver self, int dim)"""
        return _solvers.GMRESSolver_SetKDim(self, dim)
    SetKDim = _swig_new_instance_method(_solvers.GMRESSolver_SetKDim)

    def Mult(self, b, x):
        r"""Mult(GMRESSolver self, Vector b, Vector x)"""
        return _solvers.GMRESSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.GMRESSolver_Mult)
    __swig_destroy__ = _solvers.delete_GMRESSolver

# Register GMRESSolver in _solvers:
_solvers.GMRESSolver_swigregister(GMRESSolver)

class FGMRESSolver(IterativeSolver):
    r"""Proxy of C++ mfem::FGMRESSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(FGMRESSolver self) -> FGMRESSolver
        __init__(FGMRESSolver self, MPI_Comm _comm) -> FGMRESSolver
        """
        _solvers.FGMRESSolver_swiginit(self, _solvers.new_FGMRESSolver(*args))

    def SetKDim(self, dim):
        r"""SetKDim(FGMRESSolver self, int dim)"""
        return _solvers.FGMRESSolver_SetKDim(self, dim)
    SetKDim = _swig_new_instance_method(_solvers.FGMRESSolver_SetKDim)

    def Mult(self, b, x):
        r"""Mult(FGMRESSolver self, Vector b, Vector x)"""
        return _solvers.FGMRESSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.FGMRESSolver_Mult)
    __swig_destroy__ = _solvers.delete_FGMRESSolver

# Register FGMRESSolver in _solvers:
_solvers.FGMRESSolver_swigregister(FGMRESSolver)


def GMRES(*args):
    r"""
    GMRES(Operator A, Vector x, Vector b, Solver M, int & max_iter, int m, double & tol, double atol, int printit) -> int
    GMRES(Operator A, Solver B, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, int m=50, double rtol=1e-12, double atol=1e-24)
    """
    return _solvers.GMRES(*args)
GMRES = _solvers.GMRES
class BiCGSTABSolver(IterativeSolver):
    r"""Proxy of C++ mfem::BiCGSTABSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(BiCGSTABSolver self) -> BiCGSTABSolver
        __init__(BiCGSTABSolver self, MPI_Comm _comm) -> BiCGSTABSolver
        """
        _solvers.BiCGSTABSolver_swiginit(self, _solvers.new_BiCGSTABSolver(*args))

    def SetOperator(self, op):
        r"""SetOperator(BiCGSTABSolver self, Operator op)"""
        return _solvers.BiCGSTABSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.BiCGSTABSolver_SetOperator)

    def Mult(self, b, x):
        r"""Mult(BiCGSTABSolver self, Vector b, Vector x)"""
        return _solvers.BiCGSTABSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.BiCGSTABSolver_Mult)
    __swig_destroy__ = _solvers.delete_BiCGSTABSolver

# Register BiCGSTABSolver in _solvers:
_solvers.BiCGSTABSolver_swigregister(BiCGSTABSolver)


def BiCGSTAB(*args):
    r"""
    BiCGSTAB(Operator A, Vector x, Vector b, Solver M, int & max_iter, double & tol, double atol, int printit) -> int
    BiCGSTAB(Operator A, Solver B, Vector b, Vector x, int print_iter=0, int max_num_iter=1000, double rtol=1e-12, double atol=1e-24)
    """
    return _solvers.BiCGSTAB(*args)
BiCGSTAB = _solvers.BiCGSTAB
class MINRESSolver(IterativeSolver):
    r"""Proxy of C++ mfem::MINRESSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(MINRESSolver self) -> MINRESSolver
        __init__(MINRESSolver self, MPI_Comm _comm) -> MINRESSolver
        """
        _solvers.MINRESSolver_swiginit(self, _solvers.new_MINRESSolver(*args))

    def SetPreconditioner(self, pr):
        r"""SetPreconditioner(MINRESSolver self, Solver pr)"""
        return _solvers.MINRESSolver_SetPreconditioner(self, pr)
    SetPreconditioner = _swig_new_instance_method(_solvers.MINRESSolver_SetPreconditioner)

    def SetOperator(self, op):
        r"""SetOperator(MINRESSolver self, Operator op)"""
        return _solvers.MINRESSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.MINRESSolver_SetOperator)

    def Mult(self, b, x):
        r"""Mult(MINRESSolver self, Vector b, Vector x)"""
        return _solvers.MINRESSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.MINRESSolver_Mult)
    __swig_destroy__ = _solvers.delete_MINRESSolver

# Register MINRESSolver in _solvers:
_solvers.MINRESSolver_swigregister(MINRESSolver)


def MINRES(*args):
    r"""
    MINRES(Operator A, Vector b, Vector x, int print_it=0, int max_it=1000, double rtol=1e-12, double atol=1e-24)
    MINRES(Operator A, Solver B, Vector b, Vector x, int print_it=0, int max_it=1000, double rtol=1e-12, double atol=1e-24)
    """
    return _solvers.MINRES(*args)
MINRES = _solvers.MINRES
class NewtonSolver(IterativeSolver):
    r"""Proxy of C++ mfem::NewtonSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(NewtonSolver self) -> NewtonSolver
        __init__(NewtonSolver self, MPI_Comm _comm) -> NewtonSolver
        """
        _solvers.NewtonSolver_swiginit(self, _solvers.new_NewtonSolver(*args))

    def SetOperator(self, op):
        r"""SetOperator(NewtonSolver self, Operator op)"""
        return _solvers.NewtonSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.NewtonSolver_SetOperator)

    def SetSolver(self, solver):
        r"""SetSolver(NewtonSolver self, Solver solver)"""
        return _solvers.NewtonSolver_SetSolver(self, solver)
    SetSolver = _swig_new_instance_method(_solvers.NewtonSolver_SetSolver)

    def Mult(self, b, x):
        r"""Mult(NewtonSolver self, Vector b, Vector x)"""
        return _solvers.NewtonSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.NewtonSolver_Mult)

    def ComputeScalingFactor(self, x, b):
        r"""ComputeScalingFactor(NewtonSolver self, Vector x, Vector b) -> double"""
        return _solvers.NewtonSolver_ComputeScalingFactor(self, x, b)
    ComputeScalingFactor = _swig_new_instance_method(_solvers.NewtonSolver_ComputeScalingFactor)

    def ProcessNewState(self, x):
        r"""ProcessNewState(NewtonSolver self, Vector x)"""
        return _solvers.NewtonSolver_ProcessNewState(self, x)
    ProcessNewState = _swig_new_instance_method(_solvers.NewtonSolver_ProcessNewState)
    __swig_destroy__ = _solvers.delete_NewtonSolver

# Register NewtonSolver in _solvers:
_solvers.NewtonSolver_swigregister(NewtonSolver)

class LBFGSSolver(NewtonSolver):
    r"""Proxy of C++ mfem::LBFGSSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(LBFGSSolver self) -> LBFGSSolver
        __init__(LBFGSSolver self, MPI_Comm _comm) -> LBFGSSolver
        """
        _solvers.LBFGSSolver_swiginit(self, _solvers.new_LBFGSSolver(*args))

    def SetHistorySize(self, dim):
        r"""SetHistorySize(LBFGSSolver self, int dim)"""
        return _solvers.LBFGSSolver_SetHistorySize(self, dim)
    SetHistorySize = _swig_new_instance_method(_solvers.LBFGSSolver_SetHistorySize)

    def Mult(self, b, x):
        r"""Mult(LBFGSSolver self, Vector b, Vector x)"""
        return _solvers.LBFGSSolver_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.LBFGSSolver_Mult)

    def SetPreconditioner(self, pr):
        r"""SetPreconditioner(LBFGSSolver self, Solver pr)"""
        return _solvers.LBFGSSolver_SetPreconditioner(self, pr)
    SetPreconditioner = _swig_new_instance_method(_solvers.LBFGSSolver_SetPreconditioner)

    def SetSolver(self, solver):
        r"""SetSolver(LBFGSSolver self, Solver solver)"""
        return _solvers.LBFGSSolver_SetSolver(self, solver)
    SetSolver = _swig_new_instance_method(_solvers.LBFGSSolver_SetSolver)
    __swig_destroy__ = _solvers.delete_LBFGSSolver

# Register LBFGSSolver in _solvers:
_solvers.LBFGSSolver_swigregister(LBFGSSolver)


def aGMRES(A, x, b, M, max_iter, m_max, m_min, m_step, cf, tol, atol, printit):
    r"""aGMRES(Operator A, Vector x, Vector b, Operator M, int & max_iter, int m_max, int m_min, int m_step, double cf, double & tol, double & atol, int printit) -> int"""
    return _solvers.aGMRES(A, x, b, M, max_iter, m_max, m_min, m_step, cf, tol, atol, printit)
aGMRES = _solvers.aGMRES
class OptimizationProblem(object):
    r"""Proxy of C++ mfem::OptimizationProblem class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    input_size = property(_solvers.OptimizationProblem_input_size_get, doc=r"""input_size : q(const).int""")

    def CalcObjective(self, x):
        r"""CalcObjective(OptimizationProblem self, Vector x) -> double"""
        return _solvers.OptimizationProblem_CalcObjective(self, x)
    CalcObjective = _swig_new_instance_method(_solvers.OptimizationProblem_CalcObjective)

    def CalcObjectiveGrad(self, x, grad):
        r"""CalcObjectiveGrad(OptimizationProblem self, Vector x, Vector grad)"""
        return _solvers.OptimizationProblem_CalcObjectiveGrad(self, x, grad)
    CalcObjectiveGrad = _swig_new_instance_method(_solvers.OptimizationProblem_CalcObjectiveGrad)

    def SetEqualityConstraint(self, c):
        r"""SetEqualityConstraint(OptimizationProblem self, Vector c)"""
        return _solvers.OptimizationProblem_SetEqualityConstraint(self, c)
    SetEqualityConstraint = _swig_new_instance_method(_solvers.OptimizationProblem_SetEqualityConstraint)

    def SetInequalityConstraint(self, dl, dh):
        r"""SetInequalityConstraint(OptimizationProblem self, Vector dl, Vector dh)"""
        return _solvers.OptimizationProblem_SetInequalityConstraint(self, dl, dh)
    SetInequalityConstraint = _swig_new_instance_method(_solvers.OptimizationProblem_SetInequalityConstraint)

    def SetSolutionBounds(self, xl, xh):
        r"""SetSolutionBounds(OptimizationProblem self, Vector xl, Vector xh)"""
        return _solvers.OptimizationProblem_SetSolutionBounds(self, xl, xh)
    SetSolutionBounds = _swig_new_instance_method(_solvers.OptimizationProblem_SetSolutionBounds)

    def GetC(self):
        r"""GetC(OptimizationProblem self) -> Operator"""
        return _solvers.OptimizationProblem_GetC(self)
    GetC = _swig_new_instance_method(_solvers.OptimizationProblem_GetC)

    def GetD(self):
        r"""GetD(OptimizationProblem self) -> Operator"""
        return _solvers.OptimizationProblem_GetD(self)
    GetD = _swig_new_instance_method(_solvers.OptimizationProblem_GetD)

    def GetEqualityVec(self):
        r"""GetEqualityVec(OptimizationProblem self) -> Vector"""
        return _solvers.OptimizationProblem_GetEqualityVec(self)
    GetEqualityVec = _swig_new_instance_method(_solvers.OptimizationProblem_GetEqualityVec)

    def GetInequalityVec_Lo(self):
        r"""GetInequalityVec_Lo(OptimizationProblem self) -> Vector"""
        return _solvers.OptimizationProblem_GetInequalityVec_Lo(self)
    GetInequalityVec_Lo = _swig_new_instance_method(_solvers.OptimizationProblem_GetInequalityVec_Lo)

    def GetInequalityVec_Hi(self):
        r"""GetInequalityVec_Hi(OptimizationProblem self) -> Vector"""
        return _solvers.OptimizationProblem_GetInequalityVec_Hi(self)
    GetInequalityVec_Hi = _swig_new_instance_method(_solvers.OptimizationProblem_GetInequalityVec_Hi)

    def GetBoundsVec_Lo(self):
        r"""GetBoundsVec_Lo(OptimizationProblem self) -> Vector"""
        return _solvers.OptimizationProblem_GetBoundsVec_Lo(self)
    GetBoundsVec_Lo = _swig_new_instance_method(_solvers.OptimizationProblem_GetBoundsVec_Lo)

    def GetBoundsVec_Hi(self):
        r"""GetBoundsVec_Hi(OptimizationProblem self) -> Vector"""
        return _solvers.OptimizationProblem_GetBoundsVec_Hi(self)
    GetBoundsVec_Hi = _swig_new_instance_method(_solvers.OptimizationProblem_GetBoundsVec_Hi)

    def GetNumConstraints(self):
        r"""GetNumConstraints(OptimizationProblem self) -> int"""
        return _solvers.OptimizationProblem_GetNumConstraints(self)
    GetNumConstraints = _swig_new_instance_method(_solvers.OptimizationProblem_GetNumConstraints)
    __swig_destroy__ = _solvers.delete_OptimizationProblem

# Register OptimizationProblem in _solvers:
_solvers.OptimizationProblem_swigregister(OptimizationProblem)

class OptimizationSolver(IterativeSolver):
    r"""Proxy of C++ mfem::OptimizationSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _solvers.delete_OptimizationSolver

    def SetOptimizationProblem(self, prob):
        r"""SetOptimizationProblem(OptimizationSolver self, OptimizationProblem prob)"""
        return _solvers.OptimizationSolver_SetOptimizationProblem(self, prob)
    SetOptimizationProblem = _swig_new_instance_method(_solvers.OptimizationSolver_SetOptimizationProblem)

    def Mult(self, xt, x):
        r"""Mult(OptimizationSolver self, Vector xt, Vector x)"""
        return _solvers.OptimizationSolver_Mult(self, xt, x)
    Mult = _swig_new_instance_method(_solvers.OptimizationSolver_Mult)

    def SetPreconditioner(self, pr):
        r"""SetPreconditioner(OptimizationSolver self, Solver pr)"""
        return _solvers.OptimizationSolver_SetPreconditioner(self, pr)
    SetPreconditioner = _swig_new_instance_method(_solvers.OptimizationSolver_SetPreconditioner)

    def SetOperator(self, op):
        r"""SetOperator(OptimizationSolver self, Operator op)"""
        return _solvers.OptimizationSolver_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.OptimizationSolver_SetOperator)

# Register OptimizationSolver in _solvers:
_solvers.OptimizationSolver_swigregister(OptimizationSolver)

class SLBQPOptimizer(OptimizationSolver):
    r"""Proxy of C++ mfem::SLBQPOptimizer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(SLBQPOptimizer self) -> SLBQPOptimizer
        __init__(SLBQPOptimizer self, MPI_Comm _comm) -> SLBQPOptimizer
        """
        _solvers.SLBQPOptimizer_swiginit(self, _solvers.new_SLBQPOptimizer(*args))

    def SetOptimizationProblem(self, prob):
        r"""SetOptimizationProblem(SLBQPOptimizer self, OptimizationProblem prob)"""
        return _solvers.SLBQPOptimizer_SetOptimizationProblem(self, prob)
    SetOptimizationProblem = _swig_new_instance_method(_solvers.SLBQPOptimizer_SetOptimizationProblem)

    def SetBounds(self, _lo, _hi):
        r"""SetBounds(SLBQPOptimizer self, Vector _lo, Vector _hi)"""
        return _solvers.SLBQPOptimizer_SetBounds(self, _lo, _hi)
    SetBounds = _swig_new_instance_method(_solvers.SLBQPOptimizer_SetBounds)

    def SetLinearConstraint(self, _w, _a):
        r"""SetLinearConstraint(SLBQPOptimizer self, Vector _w, double _a)"""
        return _solvers.SLBQPOptimizer_SetLinearConstraint(self, _w, _a)
    SetLinearConstraint = _swig_new_instance_method(_solvers.SLBQPOptimizer_SetLinearConstraint)

    def Mult(self, xt, x):
        r"""Mult(SLBQPOptimizer self, Vector xt, Vector x)"""
        return _solvers.SLBQPOptimizer_Mult(self, xt, x)
    Mult = _swig_new_instance_method(_solvers.SLBQPOptimizer_Mult)
    __swig_destroy__ = _solvers.delete_SLBQPOptimizer

# Register SLBQPOptimizer in _solvers:
_solvers.SLBQPOptimizer_swigregister(SLBQPOptimizer)

class BlockILU(mfem._par.operators.Solver):
    r"""Proxy of C++ mfem::BlockILU class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Reordering_MINIMUM_DISCARDED_FILL = _solvers.BlockILU_Reordering_MINIMUM_DISCARDED_FILL
    
    Reordering_NONE = _solvers.BlockILU_Reordering_NONE
    

    def __init__(self, *args):
        r"""
        __init__(BlockILU self, int block_size_, mfem::BlockILU::Reordering reordering_=MINIMUM_DISCARDED_FILL, int k_fill_=0) -> BlockILU
        __init__(BlockILU self, Operator op, int block_size_=1, mfem::BlockILU::Reordering reordering_=MINIMUM_DISCARDED_FILL, int k_fill_=0) -> BlockILU
        """
        _solvers.BlockILU_swiginit(self, _solvers.new_BlockILU(*args))

    def SetOperator(self, op):
        r"""SetOperator(BlockILU self, Operator op)"""
        return _solvers.BlockILU_SetOperator(self, op)
    SetOperator = _swig_new_instance_method(_solvers.BlockILU_SetOperator)

    def Mult(self, b, x):
        r"""Mult(BlockILU self, Vector b, Vector x)"""
        return _solvers.BlockILU_Mult(self, b, x)
    Mult = _swig_new_instance_method(_solvers.BlockILU_Mult)

    def GetBlockI(self):
        r"""GetBlockI(BlockILU self) -> int *"""
        return _solvers.BlockILU_GetBlockI(self)
    GetBlockI = _swig_new_instance_method(_solvers.BlockILU_GetBlockI)

    def GetBlockJ(self):
        r"""GetBlockJ(BlockILU self) -> int *"""
        return _solvers.BlockILU_GetBlockJ(self)
    GetBlockJ = _swig_new_instance_method(_solvers.BlockILU_GetBlockJ)

    def GetBlockData(self):
        r"""GetBlockData(BlockILU self) -> double *"""
        return _solvers.BlockILU_GetBlockData(self)
    GetBlockData = _swig_new_instance_method(_solvers.BlockILU_GetBlockData)
    __swig_destroy__ = _solvers.delete_BlockILU

# Register BlockILU in _solvers:
_solvers.BlockILU_swigregister(BlockILU)

class ResidualBCMonitor(IterativeSolverMonitor):
    r"""Proxy of C++ mfem::ResidualBCMonitor class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, ess_dofs_list_):
        r"""__init__(ResidualBCMonitor self, intArray ess_dofs_list_) -> ResidualBCMonitor"""
        _solvers.ResidualBCMonitor_swiginit(self, _solvers.new_ResidualBCMonitor(ess_dofs_list_))

    def MonitorResidual(self, it, norm, r, final):
        r"""MonitorResidual(ResidualBCMonitor self, int it, double norm, Vector r, bool final)"""
        return _solvers.ResidualBCMonitor_MonitorResidual(self, it, norm, r, final)
    MonitorResidual = _swig_new_instance_method(_solvers.ResidualBCMonitor_MonitorResidual)
    __swig_destroy__ = _solvers.delete_ResidualBCMonitor

# Register ResidualBCMonitor in _solvers:
_solvers.ResidualBCMonitor_swigregister(ResidualBCMonitor)






'''
   MFEM example 5p

   See c++ version in the MFEM library for more detail 
'''
from mfem import path
import mfem.par as mfem
from mfem.par import intArray
from os.path import expanduser, join
from mpi4py import MPI
import numpy as np
from numpy import sin, cos, exp

#### time.clock deprecated and removed in PY3.8
import time
try:
    clock = time.process_time
except AttributeError:
    clock = time.clock

def pFunc_exact(x):
    xi = float(x[0]); yi = float(x[1]); zi = 0.0
    if len(x) == 3: zi = x[2]
    from numpy import sin, cos, exp        
    return exp(xi)*sin(yi)*cos(zi)

class uFunc_ex(mfem.VectorPyCoefficient):
   def EvalValue(self, x):
       xi = float(x[0]); yi = float(x[1]); zi = 0.0
       if len(x) == 3: zi = x[2]
       ret = [- exp(xi)*sin(yi)*cos(zi),
              - exp(xi)*cos(yi)*cos(zi)]
       if len(x) == 3:
          ret.append(exp(xi)*sin(yi)*sin(zi))
       return ret
       
class pFunc_ex(mfem.PyCoefficient):
   def EvalValue(self, x):
       return  pFunc_exact(x)
   
class fFunc(mfem.VectorPyCoefficient):
   def EvalValue(self, x):
       if len(x) == 3:
           return [0., 0., 0.]
       else:
           return [0., 0.]

class gFunc(mfem.PyCoefficient):
   def EvalValue(self, x):
       if len(x) == 3:
           return  -pFunc_exact(x)
       else:
           return 0.0
class f_natural(mfem.PyCoefficient):
   def EvalValue(self, x):    
       return -pFunc_exact(x)

num_proc = MPI.COMM_WORLD.size
myid     = MPI.COMM_WORLD.rank
verbose = (myid == 0)

order =1 
static_cond = False
meshfile = expanduser(join(path, 'data', 'star.mesh'))
mesh = mfem.Mesh(meshfile, 1,1)

dim = mesh.Dimension()

ref_levels = int(np.floor(np.log(10000./mesh.GetNE())/np.log(2.)/dim))
for x in range(ref_levels):
   mesh.UniformRefinement();

pmesh = mfem.ParMesh(MPI.COMM_WORLD, mesh)
par_ref_levels = 2
for l in range(par_ref_levels): pmesh.UniformRefinement()

hdiv_coll = mfem.RT_FECollection(order, dim)
l2_coll = mfem.L2_FECollection(order, dim)

R_space = mfem.ParFiniteElementSpace(pmesh, hdiv_coll)
W_space = mfem.ParFiniteElementSpace(pmesh, l2_coll)

dimR = R_space.GlobalTrueVSize()
dimW = W_space.GlobalTrueVSize()

if verbose:
    print("***********************************************************")
    print("dim(R) = " + str(dimR))
    print("dim(W) = " + str(dimW))
    print("dim(R+W) = " + str(dimR+dimW))
    print("***********************************************************")

block_offsets = intArray([0, R_space.GetVSize(), W_space.GetVSize()])
block_offsets.PartialSum()
block_trueOffsets = intArray([0, R_space.TrueVSize(), W_space.TrueVSize()])
block_trueOffsets.PartialSum()

k = mfem.ConstantCoefficient(1.0)

fcoeff = fFunc(dim)
fnatcoeff = f_natural()
gcoeff = gFunc()
ucoeff = uFunc_ex(dim)
pcoeff = pFunc_ex()

x = mfem.BlockVector(block_offsets)

rhs = mfem.BlockVector(block_offsets)
trueX = mfem.BlockVector(block_trueOffsets)

trueRhs = mfem.BlockVector(block_trueOffsets)
trueRhs.Assign(0.0)

fform = mfem.ParLinearForm()
fform.Update(R_space, rhs.GetBlock(0), 0)
fform.AddDomainIntegrator(mfem.VectorFEDomainLFIntegrator(fcoeff))
fform.AddBoundaryIntegrator(mfem.VectorFEBoundaryFluxLFIntegrator(fnatcoeff))
fform.Assemble();
fform.ParallelAssemble(trueRhs.GetBlock(0))

gform = mfem.ParLinearForm()
gform.Update(W_space, rhs.GetBlock(1), 0)
gform.AddDomainIntegrator(mfem.DomainLFIntegrator(gcoeff))
gform.Assemble()
gform.ParallelAssemble(trueRhs.GetBlock(1))

mVarf = mfem.ParBilinearForm(R_space)
bVarf = mfem.ParMixedBilinearForm(R_space, W_space)


mVarf.AddDomainIntegrator(mfem.VectorFEMassIntegrator(k))
mVarf.Assemble();
mVarf.Finalize();
M = mVarf.ParallelAssemble()

bVarf.AddDomainIntegrator(mfem.VectorFEDivergenceIntegrator())
bVarf.Assemble()
bVarf.Finalize()
B = bVarf.ParallelAssemble()
B *= -1;
BT = B.Transpose()

darcyOp = mfem.BlockOperator(block_trueOffsets)
darcyOp.SetBlock(0,0, M)
darcyOp.SetBlock(0,1, BT)
darcyOp.SetBlock(1,0, B)

#M2 = M.Transpose()
#M3 = M2.Transpose()
MinvBt =  B.Transpose()
Md = mfem.HypreParVector(MPI.COMM_WORLD, M.GetGlobalNumRows(),
                                         M.GetRowStarts())
M.GetDiag(Md)
MinvBt.InvScaleRows(Md)
S = mfem.hypre.ParMult(B, MinvBt)

invM = mfem.HypreDiagScale(M)
invS = mfem.HypreBoomerAMG(S)
invM.iterative_mode = False
invS.iterative_mode = False

darcyPr = mfem.BlockDiagonalPreconditioner(block_trueOffsets)
darcyPr.SetDiagonalBlock(0, invM);
darcyPr.SetDiagonalBlock(1, invS);

maxIter = 500; rtol = 1e-6; atol = 1e-10

import time
stime = clock()
solver = mfem.MINRESSolver(MPI.COMM_WORLD)

solver.SetAbsTol(atol)
solver.SetRelTol(rtol)
solver.SetMaxIter(maxIter)
solver.SetOperator(darcyOp)
solver.SetPreconditioner(darcyPr)
solver.SetPrintLevel(1)
trueX.Assign(0.0)
solver.Mult(trueRhs, trueX)

solve_time = clock() - stime

if verbose:
    if solver.GetConverged():
       print("MINRES converged in " + str(solver.GetNumIterations()) + 
          " iterations with a residual norm of " + str(solver.GetFinalNorm()))
    else:
       print("MINRES did not converge in " + str(solver.GetNumIterations()) + 
             " iterations. Residual norm is " + str(solver.GetFinalNorm()))
    print("MINRES solver took " + str(solve_time) +  "s.")

u = mfem.ParGridFunction(); p = mfem.ParGridFunction()
u.MakeRef(R_space, x.GetBlock(0), 0)
p.MakeRef(W_space, x.GetBlock(1), 0)
u.Distribute(trueX.GetBlock(0))
p.Distribute(trueX.GetBlock(1))

order_quad = max(2, 2*order+1);
irs = [mfem.IntRules.Get(i, order_quad)
       for i in range(mfem.Geometry.NumGeom)]
       
err_u  = u.ComputeL2Error(ucoeff, irs);
norm_u = mfem.ComputeGlobalLpNorm(2, ucoeff, pmesh, irs)
err_p  = p.ComputeL2Error(pcoeff, irs);
norm_p = mfem.ComputeGlobalLpNorm(2, pcoeff, pmesh, irs);

if verbose:
    print("|| u_h - u_ex || / || u_ex || = " + str(err_u / norm_u))
    print("|| p_h - p_ex || / || p_ex || = " + str(err_p / norm_p))

